/**
 * Copyright (c) 2021, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import { AlertLevels } from "@wso2is/core/src/models";
import { addAlert } from "@wso2is/core/src/store";
import React, { FunctionComponent, ReactElement, SyntheticEvent, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useDispatch } from "react-redux";
import { Checkbox, CheckboxProps, Divider, Header, Segment } from "semantic-ui-react";
import {
    GovernanceConnectorInterface,
    GovernanceConnectorUtils,
    updateGovernanceConnector
} from "../../../features/server-configurations";
import { serverConfigurationConfig } from "../../configs/server-configuration";

/**
 * Interface of the prop types of the `ExtendedDynamicConnector`.
 */
interface DynamicConnectorPropsInterface {
    connector: GovernanceConnectorInterface;
    connectorForm: ReactElement;
    connectorIllustration: string;
    connectorSubHeading: string;
    connectorToggleName: string;
}

/**
 * This component renders the connector settings.
 *
 * @param {DynamicConnectorPropsInterface} props - Component props.
 */
export const ExtendedDynamicConnector: FunctionComponent<DynamicConnectorPropsInterface> = (
    props: DynamicConnectorPropsInterface
): ReactElement => {
    const { connector, connectorForm, connectorIllustration, connectorSubHeading, connectorToggleName } = props;

    const [ showForm, setShowForm ] = useState<boolean>(false);

    const dispatch = useDispatch();
    const { t } = useTranslation();

    /**
     * Update showForm state based on existing data.
     */
    useEffect(() => {
        if (connector) {
            const enableProperty = connector.properties.find((property) => property.name === connectorToggleName);

            setShowForm(enableProperty?.value === "true");
        }
    }, [ connector ]);

    /**
     * This is called when the enable toggle changes.
     *
     * @param {SyntheticEvent} e Event object
     * @param {CheckboxProps} data The data object.
     */
    const handleToggle = (e: SyntheticEvent, data: CheckboxProps): void => {
        setShowForm(data.checked);

        const updateData = {
            operation: "UPDATE",
            properties: []
        };

        for (const property of connector.properties) {
            updateData.properties.push({
                name: GovernanceConnectorUtils.decodeConnectorPropertyName(property.name),
                value: property.value
            });
        }

        if (
            serverConfigurationConfig.connectorToggleName[ connector?.name ] &&
            serverConfigurationConfig.autoEnableConnectorToggleProperty
        ) {
            updateData.properties.push({
                name: GovernanceConnectorUtils.decodeConnectorPropertyName(
                    serverConfigurationConfig.connectorToggleName[ connector?.name ]
                ),
                value: data.checked.toString()
            });
        }

        updateGovernanceConnector(updateData, connector.categoryId, connector.id)
            .then(() => {
                dispatch(
                    addAlert({
                        description: t(
                            "console:manage.features.governanceConnectors.notifications." +
                            "updateConnector.success.description",
                            { name: connector.friendlyName }
                        ),
                        level: AlertLevels.SUCCESS,
                        message: t(
                            "console:manage.features.governanceConnectors.notifications." +
                            "updateConnector.success.message"
                        )
                    })
                );
            })
            .catch((error) => {
                if (error.response && error.response.data && error.response.data.detail) {
                    dispatch(
                        addAlert({
                            description: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.error.description",
                                { description: error.response.data.description }
                            ),
                            level: AlertLevels.ERROR,
                            message: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.error.message"
                            )
                        })
                    );
                } else {
                    // Generic error message
                    dispatch(
                        addAlert({
                            description: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.genericError.description"
                            ),
                            level: AlertLevels.ERROR,
                            message: t(
                                "console:manage.features.governanceConnectors.notifications." +
                                "updateConnector.genericError.message"
                            )
                        })
                    );
                }
            });
    };

    /**
     * This renders the enable toggle.
     */
    const connectorToggle = (): ReactElement => {
        return (
            <Checkbox
                label={ t("extensions:manage.serverConfigurations." +
                    "accountManagement.accountRecovery.toggleName") }
                toggle
                onChange={ handleToggle }
                checked={ showForm }
            />
        );
    };

    return (
        <Segment padded className="governance-connector-banner">
            <div
                className="connector-section-with-image-bg"
                style={ {
                    background: `url(${ connectorIllustration })`
                } }
            >
                <Header>
                    { t("extensions:manage.serverConfigurations.accountManagement.accountRecovery.heading") }
                    <Divider hidden />
                    <Header.Subheader>
                        { t("extensions:manage.serverConfigurations." +
                            "accountManagement.accountRecovery.subHeading") }
                    </Header.Subheader>
                </Header>
                <Divider hidden />
                { connectorToggle() }
            </div>
            { showForm && (
                <>
                    <Divider hidden />
                    <Divider />
                    <Header as="h5">
                        { t("extensions:manage.serverConfigurations.additionalSettings") }
                    </Header>
                    { connectorForm }
                </>
            ) }
        </Segment>
    );
};
